precision mediump float;
uniform sampler2D sTexture;
uniform float u_Offset;
uniform float h_Offset;

varying highp vec2 vTextureCoord;
const vec3 COEF_Y = vec3(0.299, 0.587, 0.114);
const vec3 COEF_U = vec3(-0.147, -0.289, 0.436);
const vec3 COEF_V = vec3(0.615, -0.515, -0.100);
const float UV_DIVIDE_LINE = 2.0 / 3.0;
const float UV_DIVIDE_LINE2 = 5.0 / 6.0;

void main() {

    vec2 texelOffset = vec2(u_Offset, 0.0);
    if (vTextureCoord.y <= UV_DIVIDE_LINE) {
        vec2 texCoord = vec2(vTextureCoord.x, vTextureCoord.y * 3.0 / 2.0);
        vec4 color0 = texture2D(sTexture, texCoord);
        vec4 color1 = texture2D(sTexture, texCoord + texelOffset);
        vec4 color2 = texture2D(sTexture, texCoord + texelOffset * 2.0);
        vec4 color3 = texture2D(sTexture, texCoord + texelOffset * 3.0);
        float y0 = dot(color0.rgb, COEF_Y);
        float y1 = dot(color1.rgb, COEF_Y);
        float y2 = dot(color2.rgb, COEF_Y);
        float y3 = dot(color3.rgb, COEF_Y);
        gl_FragColor = vec4(y0, y1, y2, y3);
    } else if (vTextureCoord.y <= UV_DIVIDE_LINE2) {

        float temp = step(0.5, vTextureCoord.x);
        float x = vTextureCoord.x - 0.5 * temp;
        float y = (vTextureCoord.y - UV_DIVIDE_LINE) * 6.0 + temp * h_Offset;
        vec2 texCoord = vec2(x * 2.0, y);
        vec4 color0 = texture2D(sTexture, texCoord);
        vec4 color1 = texture2D(sTexture, texCoord + texelOffset * 2.0);
        vec4 color2 = texture2D(sTexture, texCoord + texelOffset * 4.0);
        vec4 color3 = texture2D(sTexture, texCoord + texelOffset * 6.0);
        float v0 = dot(color0.rgb, COEF_U) + 0.5;
        float v1 = dot(color1.rgb, COEF_U) + 0.5;
        float v2 = dot(color2.rgb, COEF_U) + 0.5;
        float v3 = dot(color3.rgb, COEF_U) + 0.5;
        gl_FragColor = vec4(v0, v1, v2, v3);

    } else {
        float temp = step(0.5, vTextureCoord.x);
        float x = vTextureCoord.x - 0.5 * temp;
        float y = (vTextureCoord.y - UV_DIVIDE_LINE2) * 6.0 + temp * h_Offset;
        vec2 texCoord = vec2(x * 2.0, y);
        vec4 color0 = texture2D(sTexture, texCoord);
        vec4 color1 = texture2D(sTexture, texCoord + texelOffset * 2.0);
        vec4 color2 = texture2D(sTexture, texCoord + texelOffset * 4.0);
        vec4 color3 = texture2D(sTexture, texCoord + texelOffset * 6.0);
        float v0 = dot(color0.rgb, COEF_V) + 0.5;
        float v1 = dot(color1.rgb, COEF_V) + 0.5;
        float v2 = dot(color2.rgb, COEF_V) + 0.5;
        float v3 = dot(color3.rgb, COEF_V) + 0.5;
        gl_FragColor = vec4(v0, v1, v2, v3);
    }
}