precision mediump float;
uniform sampler2D sTexture;
uniform float u_Offset;

varying highp vec2 vTextureCoord;
const vec3 COEF_Y = vec3(0.299, 0.587, 0.114);
const vec3 COEF_U = vec3(-0.147, -0.289, 0.436);
const vec3 COEF_V = vec3(0.615, -0.515, -0.100);
const float UV_DIVIDE_LINE = 2.0 / 3.0;

void main() {

    vec2 texelOffset = vec2(u_Offset, 0.0);
    if (vTextureCoord.y <= UV_DIVIDE_LINE) {
        vec2 texCoord = vec2(vTextureCoord.x, vTextureCoord.y * 3.0 / 2.0);
        vec4 color0 = texture2D(sTexture, texCoord);
        vec4 color1 = texture2D(sTexture, texCoord + texelOffset);
        vec4 color2 = texture2D(sTexture, texCoord + texelOffset * 2.0);
        vec4 color3 = texture2D(sTexture, texCoord + texelOffset * 3.0);
        float y0 = dot(color0.rgb, COEF_Y);
        float y1 = dot(color1.rgb, COEF_Y);
        float y2 = dot(color2.rgb, COEF_Y);
        float y3 = dot(color3.rgb, COEF_Y);
        gl_FragColor = vec4(y0, y1, y2, y3);
    } else {
        vec2 texCoord = vec2(vTextureCoord.x, (vTextureCoord.y - UV_DIVIDE_LINE) * 3.0);
        vec4 color0 = texture2D(sTexture, texCoord);
        vec4 color1 = texture2D(sTexture, texCoord + texelOffset);
        vec4 color2 = texture2D(sTexture, texCoord + texelOffset * 2.0);
        vec4 color3 = texture2D(sTexture, texCoord + texelOffset * 3.0);
        float v0 = dot(color0.rgb, COEF_V) + 0.5;
        float u0 = dot(color1.rgb, COEF_U) + 0.5;
        float v1 = dot(color2.rgb, COEF_V) + 0.5;
        float u1 = dot(color3.rgb, COEF_U) + 0.5;
        gl_FragColor = vec4(v0, u0, v1, u1);
    }
}